==> worker-2.8.3.tar.bz2: worker-2.8.3/src/chmodop.cc <==
/* chmodop.c
 * This file belongs to Worker, a filemanager for UNIX/X11.
 * Copyright (C) 2001 Ralf Hoffmann.
 * You can contact me at: ralf.hoffmann@epost.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* $Id: chmodop.cc,v 1.20 2003/05/04 00:35:57 ralf Exp $ */

#include "chmodop.h"
#include "listermode.h"
#include "normalmode.h"
#include "worker.h"

const char *ChModOp::name="ChModOp";

ChModOp::ChModOp() : FunctionProto()
{
  onfiles = true;
  ondirs=false;
  recursive=false;
  requestflags = true;
  hasConfigure = true;
}

ChModOp::~ChModOp()
{
}

ChModOp*
ChModOp::duplicate() const
{
  ChModOp *ta=new ChModOp();
  ta->onfiles=onfiles;
  ta->ondirs=ondirs;
  ta->recursive=recursive;
  ta->requestflags=requestflags;
  return ta;
}

bool
ChModOp::isName(const char *str)
{
  if(strcmp(str,name)==0) return true; else return false;
}

const char *
ChModOp::getName()
{
  return name;
}

int
ChModOp::run(ActionMessage *msg)
{
  ListerMode *lm1;
  am=msg;
  if(msg->mode!=msg->AM_MODE_DNDACTION) {
    Lister *l1 = msg->getWorker()->getActiveLister();
    if(l1!=NULL) {
      startlister=l1;
      lm1=startlister->getActiveMode();
      if(lm1!=NULL)
        if(lm1->isType("NormalMode")==true)
          normalmodechmod();
    }
  }
  return 0;
}

bool
ChModOp::save(Datei *fh)
{
  if ( fh == NULL ) return false;
  WConfig::configPutPairBool( fh, "onfiles", onfiles );
  WConfig::configPutPairBool( fh, "ondirs", ondirs );
  WConfig::configPutPairBool( fh, "recursive", recursive );
  WConfig::configPutPairBool( fh, "requestflags", requestflags );
  return true;
}

const char *
ChModOp::getDescription()
{
  return catalog.getLocaleCom(38);
}

int
ChModOp::normalmodechmod()
{
  NormalMode *nm1=NULL;
  ListerMode *lm1=NULL;
  struct NM_specialsource *specialsource=NULL;
  bool cont=true;
  struct NM_chmodorder cmorder;
  
  if(startlister==NULL) return 1;
  lm1=startlister->getActiveMode();
  if(lm1==NULL) return 1;
  if(lm1->isType("NormalMode")!=true) return 1;
  nm1=(NormalMode*)lm1;

  if(requestflags==true) {
    if(doconfigure(1)!=0) cont=false;
  } else {
    // set values in t* variables
    tonfiles=onfiles;
    tondirs=ondirs;
    trecursive=recursive;
  }
  
  if(cont==true) {
    memset( &cmorder, 0, sizeof( cmorder ) );
    // detect what to copy
    if(am->mode==am->AM_MODE_ONLYACTIVE)
      cmorder.source=cmorder.NM_ONLYACTIVE;
    else if(am->mode==am->AM_MODE_DNDACTION) {
      // insert DND-element into list
      cmorder.source=cmorder.NM_SPECIAL;
      cmorder.sources=new List;
      specialsource=(struct NM_specialsource*)_allocsafe(sizeof(struct NM_specialsource));
      specialsource->entry=NULL;
      specialsource->row=-1;
      //TODO: specialsource nach am besetzen (je nachdem wir ich das realisiere)
      cmorder.sources->addElement(specialsource);
    } else if(am->mode==am->AM_MODE_SPECIAL) {
      cmorder.source=cmorder.NM_SPECIAL;
      cmorder.sources=new List;
      specialsource=(struct NM_specialsource*)_allocsafe(sizeof(struct NM_specialsource));
      specialsource->entry=(FileEntry*)am->extraInfo;
      specialsource->row=-1;
      cmorder.sources->addElement(specialsource);
    } else
      cmorder.source=cmorder.NM_ALLENTRIES;

    cmorder.onfiles=tonfiles;
    cmorder.ondirs=tondirs;
    cmorder.recursive=trecursive;
    nm1->chmodf(&cmorder);
    if(cmorder.source==cmorder.NM_SPECIAL) {
      if(specialsource!=NULL) _freesafe(specialsource);
      delete cmorder.sources;
    }
  }
  return 0;
}

int
ChModOp::configure()
{
  return doconfigure(0);
}

int
ChModOp::doconfigure(int mode)
{
  AGUIX *aguix = Worker::getAGUIX();
  Button *okb,*cb;
  AWindow *win;
  ChooseButton *ofcb,*odcb,*rfcb=NULL,*rcb;
  int tw,w,h,x,y,t1,t2;
  AGMessage *msg;
  int endmode=-1;
  char *tstr;
  
  w=h=10;
  x=y=5;
  win=new AWindow(aguix);
  tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(catalog.getLocaleCom(38))+1);
  sprintf(tstr,catalog.getLocale(293),catalog.getLocaleCom(38));
  win->create(NULL,10,10,w,h,0,tstr);
  _freesafe(tstr);
  
  Text *tt1 = (Text*)win->add( new Text( aguix, x, y, catalog.getLocaleCom( 38 ), 1 ) );
  y += tt1->getHeight() + 5;

  ofcb=(ChooseButton*)win->add(new ChooseButton(aguix,x,y,20,20,(onfiles==true)?1:0,
                                                catalog.getLocale(295),LABEL_RIGHT,1,0));
  y+=ofcb->getHeight()+5;
  tw=ofcb->getWidth()+10;
  if(tw>w) w=tw;

  odcb=(ChooseButton*)win->add(new ChooseButton(aguix,x,y,20,20,(ondirs==true)?1:0,
                                                catalog.getLocale(296),LABEL_RIGHT,1,0));
  y+=odcb->getHeight()+5;
  tw=odcb->getWidth()+10;
  if(tw>w) w=tw;

  rcb=(ChooseButton*)win->add(new ChooseButton(aguix,x,y,20,20,(recursive==true)?1:0,
                                                catalog.getLocale(297),LABEL_RIGHT,1,0));
  y+=rcb->getHeight()+5;
  tw=rcb->getWidth()+10;
  if(tw>w) w=tw;

  if(mode==0) {
    rfcb=(ChooseButton*)win->add(new ChooseButton(aguix,x,y,20,20,(requestflags==true)?1:0,
                                                 catalog.getLocale(294),LABEL_RIGHT,1,0));
    y+=rfcb->getHeight()+5;
    tw=rfcb->getWidth()+10;
    if(tw>w) w=tw;
  }

  t1=(strlen(catalog.getLocale(11))+2);
  t1*=aguix->getCharWidth();
  t2=(strlen(catalog.getLocale(8))+2);
  t2*=aguix->getCharWidth();
  tw=5+t1+5+t2+5;
  if(tw>w) w=tw;
  okb=(Button*)win->add(new Button(aguix,
                                   5,
                                   y,
                                   t1,
                                   catalog.getLocale(11),
                                   1,
                                   0,
                                   0));
  cb=(Button*)win->add(new Button(aguix,
                                  w-5-t2,
                                  y,
                                  t2,
                                  catalog.getLocale(8),
                                  1,
                                  0,
                                  0));
  y+=okb->getHeight()+5;
  
  h=y;
  okb->takeFocus();
  win->setDoTabCycling( true );
  win->resize(w,h);
  win->setMaxSize(w,h);
  win->setMinSize(w,h);
  win->show();
  for(;endmode==-1;) {
    msg=aguix->WaitMessage(win);
    if(msg!=NULL) {
      switch(msg->type) {
        case AG_CLOSEWINDOW:
          if(msg->closewindow.window==win->getWindow()) endmode=1;
          break;
        case AG_BUTTONCLICKED:
          if(msg->button.button==okb) endmode=0;
          else if(msg->button.button==cb) endmode=1;
          break;
        case AG_KEYPRESSED:
          if(win->isParent(msg->key.window,false)==true) {
            switch(msg->key.key) {
              case XK_1:
                ofcb->setState((ofcb->getState()==1)?0:1);
                break;
              case XK_2:
                odcb->setState((odcb->getState()==1)?0:1);
                break;
              case XK_3:
                rcb->setState((rcb->getState()==1)?0:1);
                break;
              case XK_Return:
                if ( cb->hasFocus() == false )
                  endmode=0;
                break;
              case XK_Escape:
                endmode=1;
                break;
            }
          }
          break;
      }
      aguix->ReplyMessage(msg);
    }
  }
  
  if(endmode==0) {
    // ok
    if(mode==1) {
      tonfiles=(ofcb->getState()==1)?true:false;
      tondirs=(odcb->getState()==1)?true:false;
      trecursive=(rcb->getState()==1)?true:false;
    } else {
      onfiles=(ofcb->getState()==1)?true:false;
      ondirs=(odcb->getState()==1)?true:false;
      recursive=(rcb->getState()==1)?true:false;
      requestflags=(rfcb->getState()==1)?true:false;
    }
  }
  
  win->close();
  delete win;

  return endmode;
}

void ChModOp::setOnFiles(bool nv)
{
  onfiles=nv;
}

void ChModOp::setOnDirs(bool nv)
{
  ondirs=nv;
}

void ChModOp::setRecursive(bool nv)
{
  recursive=nv;
}

void ChModOp::setRequestFlags(bool nv)
{
  requestflags=nv;
}

==> worker-2.13.1.tar.bz2: worker-2.13.1/src/chmodop.cc <==
/* chmodop.c
 * This file belongs to Worker, a filemanager for UNIX/X11.
 * Copyright (C) 2001-2004 Ralf Hoffmann.
 * You can contact me at: ralf@boomerangsworld.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* $Id: chmodop.cc,v 1.26 2005/10/19 18:07:22 ralf Exp $ */

#include "chmodop.h"
#include "listermode.h"
#include "normalmode.h"
#include "worker.h"

const char *ChModOp::name="ChModOp";

ChModOp::ChModOp() : FunctionProto()
{
  onfiles = true;
  ondirs=false;
  recursive=false;
  requestflags = true;
  hasConfigure = true;
}

ChModOp::~ChModOp()
{
}

ChModOp*
ChModOp::duplicate() const
{
  ChModOp *ta=new ChModOp();
  ta->onfiles=onfiles;
  ta->ondirs=ondirs;
  ta->recursive=recursive;
  ta->requestflags=requestflags;
  return ta;
}

bool
ChModOp::isName(const char *str)
{
  if(strcmp(str,name)==0) return true; else return false;
}

const char *
ChModOp::getName()
{
  return name;
}

int
ChModOp::run( WPUContext *wpu, ActionMessage *msg )
{
  ListerMode *lm1;
  if(msg->mode!=msg->AM_MODE_DNDACTION) {
    Lister *l1 = msg->getWorker()->getActiveLister();
    if(l1!=NULL) {
      startlister=l1;
      lm1=startlister->getActiveMode();
      if(lm1!=NULL)
        if(lm1->isType("NormalMode")==true)
          normalmodechmod( msg );
    }
  }
  return 0;
}

bool
ChModOp::save(Datei *fh)
{
  if ( fh == NULL ) return false;
  fh->configPutPairBool( "onfiles", onfiles );
  fh->configPutPairBool( "ondirs", ondirs );
  fh->configPutPairBool( "recursive", recursive );
  fh->configPutPairBool( "requestflags", requestflags );
  return true;
}

const char *
ChModOp::getDescription()
{
  return catalog.getLocaleCom(38);
}

int
ChModOp::normalmodechmod( ActionMessage *am )
{
  NormalMode *nm1=NULL;
  ListerMode *lm1=NULL;
  NM_specialsourceExt *specialsource = NULL;
  bool cont=true;
  struct NM_chmodorder cmorder;
  
  if(startlister==NULL) return 1;
  lm1=startlister->getActiveMode();
  if(lm1==NULL) return 1;
  if(lm1->isType("NormalMode")!=true) return 1;
  nm1=(NormalMode*)lm1;

  if(requestflags==true) {
    if(doconfigure(1)!=0) cont=false;
  } else {
    // set values in t* variables
    tonfiles=onfiles;
    tondirs=ondirs;
    trecursive=recursive;
  }
  
  if(cont==true) {
    memset( &cmorder, 0, sizeof( cmorder ) );
    // detect what to copy
    if(am->mode==am->AM_MODE_ONLYACTIVE)
      cmorder.source=cmorder.NM_ONLYACTIVE;
    else if(am->mode==am->AM_MODE_DNDACTION) {
      // insert DND-element into list
      cmorder.source=cmorder.NM_SPECIAL;
      cmorder.sources=new std::list<NM_specialsourceExt*>;
      specialsource = new NM_specialsourceExt( NULL );
      specialsource->row=-1;
      //TODO: specialsource nach am besetzen (je nachdem wir ich das realisiere)
      cmorder.sources->push_back( specialsource );
    } else if(am->mode==am->AM_MODE_SPECIAL) {
      cmorder.source=cmorder.NM_SPECIAL;
      cmorder.sources=new std::list<NM_specialsourceExt*>;
      specialsource = new NM_specialsourceExt( am->getFE() );
      specialsource->row=-1;
      cmorder.sources->push_back( specialsource );
    } else
      cmorder.source=cmorder.NM_ALLENTRIES;

    cmorder.onfiles=tonfiles;
    cmorder.ondirs=tondirs;
    cmorder.recursive=trecursive;
    nm1->chmodf(&cmorder);
    if(cmorder.source==cmorder.NM_SPECIAL) {
      if ( specialsource != NULL ) delete specialsource;
      delete cmorder.sources;
    }
  }
  return 0;
}

int
ChModOp::configure()
{
  return doconfigure(0);
}

int
ChModOp::doconfigure(int mode)
{
  AGUIX *aguix = Worker::getAGUIX();
  AWindow *win;
  ChooseButton *ofcb,*odcb,*rfcb=NULL,*rcb;
  AGMessage *msg;
  int endmode=-1;
  char *tstr;
  const int cincw = AContainer::ACONT_MINH +
                    AContainer::ACONT_MINW +
                    AContainer::ACONT_MAXH;
  const int cincwnr = cincw +
                      AContainer::ACONT_NORESIZE;
  const int cfix = AContainer::ACONT_MINH +
                   AContainer::ACONT_MINW +
                   AContainer::ACONT_MAXH +
                   AContainer::ACONT_MAXW;
  
  tstr=(char*)_allocsafe(strlen(catalog.getLocale(293))+strlen(catalog.getLocaleCom(38))+1);
  sprintf(tstr,catalog.getLocale(293),catalog.getLocaleCom(38));
  win = new AWindow( aguix, 10, 10, 10, 10, 0, tstr );
  win->create();
  _freesafe(tstr);
  
  AContainer *ac1 = win->setContainer( new AContainer( win, 1, 6 ), true );
  ac1->setMinSpace( 5 );
  ac1->setMaxSpace( 5 );

  ac1->add( new Text( aguix, 0, 0, catalog.getLocaleCom( 38 ), 1 ), 0, 0, cincwnr );

  ofcb = (ChooseButton*)ac1->add( new ChooseButton( aguix, 0, 0, 20, 20, ( onfiles == true ) ? 1 : 0,
						    catalog.getLocale( 295 ), LABEL_RIGHT, 1, 0 ), 0, 1, cincwnr );

  odcb = (ChooseButton*)ac1->add( new ChooseButton( aguix, 0, 0, 20, 20, ( ondirs == true ) ? 1 : 0,
						    catalog.getLocale( 296 ), LABEL_RIGHT, 1, 0 ), 0, 2, cincwnr );

  rcb = (ChooseButton*)ac1->add( new ChooseButton( aguix, 0, 0, 20, 20, ( recursive == true ) ? 1 : 0,
						   catalog.getLocale( 297 ), LABEL_RIGHT, 1, 0 ), 0, 3, cincwnr );

  if(mode==0) {
    rfcb = (ChooseButton*)ac1->add( new ChooseButton( aguix, 0, 0, 20, 20, ( requestflags == true ) ? 1 : 0,
						      catalog.getLocale( 294 ), LABEL_RIGHT, 1, 0 ), 0, 4, cincwnr );
  }

  AContainer *ac1_1 = ac1->add( new AContainer( win, 2, 1 ), 0, 5 );
  ac1_1->setMinSpace( 5 );
  ac1_1->setMaxSpace( -1 );
  ac1_1->setBorderWidth( 0 );
  Button *okb =(Button*)ac1_1->add( new Button( aguix,
                                                0,
                                                0,
                                                catalog.getLocale( 11 ),
                                                1,
                                                0,
                                                0 ), 0, 0, cfix );
  Button *cb = (Button*)ac1_1->add( new Button( aguix,
						0,
						0,
						catalog.getLocale( 8 ),
						1,
						0,
						0 ), 1, 0, cfix );
  
  okb->takeFocus();
  win->setDoTabCycling( true );
  win->contMaximize( true );
  win->show();
  for(;endmode==-1;) {
    msg=aguix->WaitMessage(win);
    if(msg!=NULL) {
      switch(msg->type) {
        case AG_CLOSEWINDOW:
          if(msg->closewindow.window==win->getWindow()) endmode=1;
          break;
        case AG_BUTTONCLICKED:
          if(msg->button.button==okb) endmode=0;
          else if(msg->button.button==cb) endmode=1;
          break;
        case AG_KEYPRESSED:
          if(win->isParent(msg->key.window,false)==true) {
            switch(msg->key.key) {
              case XK_1:
                ofcb->setState((ofcb->getState()==1)?0:1);
                break;
              case XK_2:
                odcb->setState((odcb->getState()==1)?0:1);
                break;
              case XK_3:
                rcb->setState((rcb->getState()==1)?0:1);
                break;
              case XK_Return:
                if ( cb->getHasFocus() == false )
                  endmode=0;
                break;
              case XK_Escape:
                endmode=1;
                break;
            }
          }
          break;
      }
      aguix->ReplyMessage(msg);
    }
  }
  
  if(endmode==0) {
    // ok
    if(mode==1) {
      tonfiles=(ofcb->getState()==1)?true:false;
      tondirs=(odcb->getState()==1)?true:false;
      trecursive=(rcb->getState()==1)?true:false;
    } else {
      onfiles=(ofcb->getState()==1)?true:false;
      ondirs=(odcb->getState()==1)?true:false;
      recursive=(rcb->getState()==1)?true:false;
      requestflags=(rfcb->getState()==1)?true:false;
    }
  }
  
  delete win;

  return endmode;
}

void ChModOp::setOnFiles(bool nv)
{
  onfiles=nv;
}

void ChModOp::setOnDirs(bool nv)
{
  ondirs=nv;
}

void ChModOp::setRecursive(bool nv)
{
  recursive=nv;
}

void ChModOp::setRequestFlags(bool nv)
{
  requestflags=nv;
}
